class FromHyraxController < Spotlight::ApplicationController
  layout 'spotlight/spotlight'
  before_action :authenticate_user!, except: :receive
  protect_from_forgery with: :exception
  skip_before_action :verify_authenticity_token, only: :receive
  load_and_authorize_resource :current_exhibit, class: Spotlight::Exhibit
  require 'net/http'

  def hyrax_api
    return nil unless instances.include?(params[:repo])
    uri      = URI("#{params[:repo]}/to_spotlight/api/#{params[:api_type]}")
    response = get_response(uri)
    render html: response.body.html_safe
  end

  def index
    instances
  end

  def new
    uri               = URI("#{params[:repo]}/to_spotlight/api/fields")
    response          = get_response(uri)
    @hyrax_fields     = eval(response.body)
    @spotlight_fields = spotlight_fields.map { |f| [f[:label], f[:field_name]] }
  end

  def create
    return unless instances.include? params[:repo]
    # go to: to_spotlight/receive
    @mappings    = params[:transfer][:mappings].permit!.to_h.map { |_, v| v unless v['spotlight']['0'].empty? }.compact #use hash.values
    post_options = {transfer: params[:transfer], mappings: @mappings, token: token}
    @res         = post_response URI("#{params[:repo]}/to_spotlight/receive"), post_options
    redirect_to from_hyrax_index_path
  end

  def receive
    return unless params[:token] == token
    FromHyrax::RunImportJob.perform_later(params[:csv_url], csv_params)
  end

  private

    def get_response uri
      http             = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl     = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      request          = Net::HTTP::Get.new(uri.request_uri)
      http.request(request)
    end

    def post_response uri, options = {}
      http             = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl     = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      request          = Net::HTTP::Post.new(uri.request_uri, 'Content-Type' => 'application/json')
      request.body     = options.to_json
      http.request(request)
    end

    def instances
      @instances ||= FromHyrax::Engine.config['hyrax_instances'] || []
    end

    def token
      FromHyrax::Engine.config['secret_token']
    end

    def csv_params
      params.require(:transfer).permit(:user, :exhibit_id).to_h
    end

    def spotlight_fields
      @spotlight_fields ||= Spotlight::Engine.config.upload_fields
                                .map do |f|
        {
            label:      f.label,
            field_name: f.field_name
        }
      end
    end

end
